/************************************************************************/
/*                                                                      */
/*             ,     ..        */
/*                   ,   ,   .               */
/*                                                                      */
/************************************************************************/

#include <htc.h>
#include "eeprom.h"
#include "i2c.h"

/************************************************************************/
/* AT24C512B serial EEPROM routines                                     */
/************************************************************************/

/************************************************************************/
/* Device address structure:                                            */
/* +----+----+----+----+-----+-----+----+----+                          */
/* |1   |0   |1   |0   |A2   |A1   |A0  |R/W |                          */
/* +----+----+----+----+-----+-----+----+----+                          */
/*  MSB                                  LSB                            */
/************************************************************************/

// Initiates protocol reset sequence. 
void eeprom_reset()
{
	// initiate I2C bus start condition
	i2c_start();
	// write dummy byte
	i2c_write(0);
	// initiate I2C bus start condition
	i2c_start();
	// initiate I2C bus stop condition
	i2c_stop();
}	

// Writes single byte to EEPROM.
// Returns 1 if successful, 0 otherwise.
int eeprom_write_byte(unsigned char dev_addr,
					  unsigned short word_addr, 
					  unsigned char data)
{
	// initiate I2C bus start condition
	i2c_start();
	// write device address
	if (!i2c_write(dev_addr)) goto error;
	// write first word address
	if (!i2c_write(word_addr >> 8)) goto error;
	// write second word address
	if (!i2c_write(word_addr & 0xFF)) goto error;
	// write data byte
	if (!i2c_write(data)) goto error;
	// initiate I2C bus stop condition
	i2c_stop();

	return 1;
error:
	// initiate I2C bus stop condition
	i2c_stop();
	return 0;
}

// Reads single byte from EEPROM at specified address.
// Returns 1 if successful, 0 otherwise.
int eeprom_read_random(unsigned char dev_addr,
					   unsigned short word_addr, 
					   unsigned char* data)
{
	// initiate I2C bus start condition
	i2c_start();
	// write device address
	if (!i2c_write(dev_addr)) goto error;
	// write first word address
	if (!i2c_write(word_addr >> 8)) goto error;
	// write second word address
	if (!i2c_write(word_addr & 0xFF)) goto error;
	// initiate I2C bus restart condition
	i2c_restart();
	// write device address with read command
	if (!i2c_write(dev_addr | 1)) goto error;
	// read a byte
	*data = i2c_read();
	// initiate I2C bus NOT-acknowledge condition
	i2c_nack();
	// initiate I2C bus stop condition
	i2c_stop();

	return 1;
error:
	// initiate I2C bus stop condition
	i2c_stop();
	return 0;
}

// Reads single byte from EEPROM at current address.
// Returns 1 if successful, 0 otherwise.
int eeprom_read_current(unsigned char dev_addr,
					    unsigned char* data)
{
	// initiate I2C bus start condition
	i2c_start();
	// write device address
	if (!i2c_write(dev_addr | 1)) goto error;
	// read a byte
	*data = i2c_read();
	// initiate I2C bus NOT-acknowledge condition
	i2c_nack();
	// initiate I2C bus stop condition
	i2c_stop();

	return 1;
error:
	// initiate I2C bus stop condition
	i2c_stop();
	return 0;
}
